CREATE OR REPLACE PROCEDURE RAZAO_AUX_FIL(
   P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
   P_CDFILIAL IN VARCHAR2,
   P_CDCONTINI IN VARCHAR2,
   P_CDCONTFIN IN VARCHAR2,
   P_CDCLIENTE IN VARCHAR2,
   P_CDFORNECED IN VARCHAR2,
   P_CDREDUCONT IN VARCHAR2,
   P_DTINI IN LANCCONT.DTLANCCONT%TYPE,
   P_DTFIN IN LANCCONT.DTLANCCONT%TYPE,
   P_NRRAZAO OUT NUMBER
)
AS
   V_NOVOCODIGO VARCHAR2(20);
   V_DATAINI DATE;
   V_NRRAZAO NUMBER;
   V_NRSEQ NUMBER;
   V_COUNT NUMBER;
   V_VRSALDATUAL NUMBER;
   V_VRDEBITO NUMBER;
   V_VRCREDITO NUMBER;
   V_CDCONTCOME VARCHAR2(21);
   V_CDCONTCTBL VARCHAR2(20);
   V_CDFILIAL VARCHAR2(04);
   V_IDTIPOCOME VARCHAR2(01);
   V_VRSALDANT NUMBER;
   
   FUNCTION SEQUENCIAL RETURN NUMBER
   IS
      V_TEMP VARCHAR2(20);
   BEGIN
      NOVO_CODIGO_NT('SEQRAZAOCOMFIL', V_TEMP);
      RETURN(TO_NUMBER(V_TEMP));
   END;
BEGIN
   NOVO_CODIGO_NT('RAZAOCOMFIL', V_NOVOCODIGO);
   V_NRRAZAO := TO_NUMBER(V_NOVOCODIGO);
   V_NRSEQ := SEQUENCIAL;
   BEGIN
      SELECT MIN(DTCOMPCUSTO)
        INTO V_DATAINI
        FROM LANCCCUSTO
       WHERE CDEMPRESA = P_CDEMPRESA;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_DATAINI := TO_DATE('01/01/2000');
   END;

   /* CURSOIR RESPONSVEL POR GERAR OS SALDOS ANTERIORES.
      UNIONS FORAM USADOS PARA SEPARAR OS SALDOS POR TIPO COMERCIAL: CLIENTE, FORNECEDOR E BANCO */
   FOR C_SALDOS IN
      (SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, SUM(LC.VRLANCCUST) VRSALDANT, 'C' IDTIPO
         FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                            FROM FILIAL F
                                           WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
        WHERE LC.CDEMPRESA = P_CDEMPRESA
          AND LC.CDEMPRESA = L.CDEMPRESA
          AND LC.NRLANCCONT = L.NRLANCCONT
          AND LC.NRSEQLANC = L.NRSEQLANC
          AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
          AND LC.DTCOMPCUSTO BETWEEN V_DATAINI AND (P_DTINI-1)
          AND LC.CDFILIAL = F.CDFILIAL
          AND L.IDLANCCOME = 'C'
          AND L.CDCONTCOME IS NOT NULL
          AND ((INSTR(P_CDCLIENTE, L.CDCONTCOME) > 0) OR ('T' = P_CDCLIENTE))
        GROUP BY LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME
       HAVING SUM(LC.VRLANCCUST) <> 0

        UNION ALL

       SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, SUM(LC.VRLANCCUST) VRSALDANT, 'F' IDTIPO
         FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                            FROM FILIAL F
                                           WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
        WHERE LC.CDEMPRESA = P_CDEMPRESA
          AND LC.CDEMPRESA = L.CDEMPRESA
          AND LC.NRLANCCONT = L.NRLANCCONT
          AND LC.NRSEQLANC = L.NRSEQLANC
          AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
          AND LC.DTCOMPCUSTO BETWEEN V_DATAINI AND (P_DTINI-1)
          AND LC.CDFILIAL = F.CDFILIAL
          AND L.IDLANCCOME = 'F'
          AND L.CDCONTCOME IS NOT NULL
          AND ((INSTR(P_CDFORNECED, L.CDCONTCOME) > 0) OR ('T' = P_CDFORNECED))
        GROUP BY LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME
       HAVING SUM(LC.VRLANCCUST) <> 0

        UNION ALL

       SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, SUM(LC.VRLANCCUST) VRSALDANT, 'B' IDTIPO
         FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                            FROM FILIAL F
                                           WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
        WHERE LC.CDEMPRESA = P_CDEMPRESA
          AND LC.CDEMPRESA = L.CDEMPRESA
          AND LC.NRLANCCONT = L.NRLANCCONT
          AND LC.NRSEQLANC = L.NRSEQLANC
          AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
          AND LC.DTCOMPCUSTO BETWEEN V_DATAINI AND (P_DTINI-1)
          AND LC.CDFILIAL = F.CDFILIAL
          AND L.IDLANCCOME = 'B'
          AND L.CDCONTCOME IS NOT NULL
          AND ((INSTR(P_CDREDUCONT, L.CDCONTCOME) > 0) OR ('T' = P_CDREDUCONT))
        GROUP BY LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME
       HAVING SUM(LC.VRLANCCUST) <> 0) LOOP

      V_NRSEQ := SEQUENCIAL;
      INSERT INTO RAZAOCOMFIL
         (NRRAZCOMFIL, NRSEQRAZCOM, CDEMPRESA, DTCOMPCUSTO, NRLANCCONT, NRSEQLANC, CDFILIAL,
          CDCONTCTBL, CDCONTCOME, IDREGISTRO, IDTIPOCOME, VRSALDANT,
          VRDEBITO, VRCREDITO, VRSALDATUAL)
      VALUES
          (V_NRRAZAO, V_NRSEQ, P_CDEMPRESA, NULL, NULL, NULL, C_SALDOS.CDFILIAL,
           C_SALDOS.CDCONTCTBL, C_SALDOS.CDCONTCOME, 'S', C_SALDOS.IDTIPO, C_SALDOS.VRSALDANT,
           0, 0, C_SALDOS.VRSALDANT);
   END LOOP;

   V_CDCONTCTBL := 'XXXX';
   V_CDFILIAL := 'XXXX';
   V_CDCONTCOME := 'XXXX';
   V_IDTIPOCOME := 'X';

   /* CURSOR USADO PARA PEGAR OS LANAMENTOS DE DBITO E CRDITO. */
   FOR C_LANC IN
      ( SELECT * FROM
         (SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, LC.NRLANCCONT,
                 LC.NRSEQLANC, LC.DTCOMPCUSTO, LC.VRLANCCUST,
                 L.DSHISTLANC, 'C' IDTIPO, '1' ORDEM
            FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                               FROM FILIAL F
                                              WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
           WHERE LC.CDEMPRESA = P_CDEMPRESA
             AND LC.CDEMPRESA = L.CDEMPRESA
             AND LC.NRLANCCONT = L.NRLANCCONT
             AND LC.NRSEQLANC = L.NRSEQLANC
             AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
             AND LC.DTCOMPCUSTO BETWEEN P_DTINI AND P_DTFIN
             AND LC.CDFILIAL = F.CDFILIAL
             AND L.IDLANCCOME = 'C'
             AND L.CDCONTCOME IS NOT NULL
             AND ((INSTR(P_CDCLIENTE, L.CDCONTCOME) > 0) OR ('T' = P_CDCLIENTE))

           UNION ALL

          SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, LC.NRLANCCONT,
                 LC.NRSEQLANC, LC.DTCOMPCUSTO, LC.VRLANCCUST, L.DSHISTLANC, 'F' IDTIPO, '2' ORDEM
            FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                               FROM FILIAL F
                                              WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
           WHERE LC.CDEMPRESA = P_CDEMPRESA
             AND LC.CDEMPRESA = L.CDEMPRESA
             AND LC.NRLANCCONT = L.NRLANCCONT
             AND LC.NRSEQLANC = L.NRSEQLANC
             AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
             AND LC.DTCOMPCUSTO BETWEEN P_DTINI AND P_DTFIN
             AND LC.CDFILIAL = F.CDFILIAL
             AND L.IDLANCCOME = 'F'
             AND L.CDCONTCOME IS NOT NULL
             AND ((INSTR(P_CDFORNECED, L.CDCONTCOME) > 0) OR ('T' = P_CDFORNECED))

           UNION ALL

          SELECT LC.CDEMPRESA, LC.CDFILIAL, L.CDCONTCTBL, L.CDCONTCOME, LC.NRLANCCONT,
                 LC.NRSEQLANC, LC.DTCOMPCUSTO, LC.VRLANCCUST, L.DSHISTLANC, 'B' IDTIPO, '3' ORDEM
            FROM LANCCCUSTO LC, LANCCONT L, (SELECT CDFILIAL
                                               FROM FILIAL F
                                              WHERE ((INSTR(P_CDFILIAL, CDFILIAL) > 0) OR ('T' = P_CDFILIAL))) F
           WHERE LC.CDEMPRESA = P_CDEMPRESA
             AND LC.CDEMPRESA = L.CDEMPRESA
             AND LC.NRLANCCONT = L.NRLANCCONT
             AND LC.NRSEQLANC = L.NRSEQLANC
             AND LC.CDCONTCTBL BETWEEN P_CDCONTINI AND P_CDCONTFIN
             AND LC.DTCOMPCUSTO BETWEEN P_DTINI AND P_DTFIN
             AND LC.CDFILIAL = F.CDFILIAL
             AND L.IDLANCCOME = 'B'
             AND L.CDCONTCOME IS NOT NULL
             AND ((INSTR(P_CDREDUCONT, L.CDCONTCOME) > 0) OR ('T' = P_CDREDUCONT)))
          ORDER BY CDFILIAL, ORDEM, CDCONTCTBL, CDCONTCOME,
                   DTCOMPCUSTO, NRLANCCONT, NRSEQLANC)LOOP

      /* VERIFICAO SE A CONTA OU CDIGO COMERCIAL FORAM TROCADOS. CASO SEJAM TROCADDOS
         O VALOR DE SALDO ANTERIOR DEVE SER TROCADO */
      IF V_CDCONTCTBL <> C_LANC.CDCONTCTBL OR
         V_CDCONTCOME <> C_LANC.CDCONTCOME OR
         V_CDFILIAL <> C_LANC.CDFILIAL OR
         V_IDTIPOCOME <> C_LANC.IDTIPO THEN
         V_CDCONTCTBL := C_LANC.CDCONTCTBL;
         V_CDCONTCOME := C_LANC.CDCONTCOME;
         V_CDFILIAL := C_LANC.CDFILIAL;
         V_IDTIPOCOME := C_LANC.IDTIPO;

         V_COUNT := 0;
         V_VRSALDANT := 0;
         V_VRSALDATUAL := 0;

         /* VERIFICA SE PARA AQUELA CONTA COM O CDIGO COMERCIAL EXISTE SALDO ANTERIOR. */
         BEGIN
            SELECT COUNT(*)
              INTO V_COUNT
              FROM RAZAOCOMFIL
             WHERE NRRAZCOMFIL = V_NRRAZAO
               AND CDCONTCTBL = V_CDCONTCTBL
               AND CDCONTCOME = V_CDCONTCOME
               AND CDFILIAL = V_CDFILIAL
               AND IDTIPOCOME = V_IDTIPOCOME
               AND IDREGISTRO = 'S';
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_COUNT := 0;
         END;
         IF V_COUNT = 0 THEN
            /* CASO NO EXISTA SALDO ANTERIOR,  INSERIDO UM SALDO ANTERIOR COM O VALOR ZERADO */
            V_NRSEQ := SEQUENCIAL;
            INSERT INTO RAZAOCOMFIL
               (NRRAZCOMFIL, NRSEQRAZCOM, CDEMPRESA, DTCOMPCUSTO, NRLANCCONT, NRSEQLANC, CDFILIAL,
                CDCONTCTBL, CDCONTCOME, IDREGISTRO, IDTIPOCOME, VRSALDANT,
                VRDEBITO, VRCREDITO, VRSALDATUAL)
            VALUES
                (V_NRRAZAO, V_NRSEQ, P_CDEMPRESA, NULL, NULL, NULL, V_CDFILIAL,
                 V_CDCONTCTBL, V_CDCONTCOME, 'S', V_IDTIPOCOME, 0, 0, 0, 0);
         ELSE
            /* CASO EXISTA SALDO ANTERIOR, A ROTINA BUSCA ESTE SALDO PARA SER ATUALIZADO.
               PARTIDA A PARTIDA */
            BEGIN
               SELECT VRSALDANT
                 INTO V_VRSALDANT
                 FROM RAZAOCOMFIL
                WHERE NRRAZCOMFIL = V_NRRAZAO
                  AND CDCONTCTBL = V_CDCONTCTBL
                  AND CDCONTCOME = V_CDCONTCOME
                  AND CDFILIAL = V_CDFILIAL
                  AND IDTIPOCOME = V_IDTIPOCOME
                  AND IDREGISTRO = 'S';
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_VRSALDANT := 0;
            END;
            V_VRSALDATUAL := V_VRSALDANT;
         END IF;
      END IF;
      /* REALIZA A SOMA DO SALDO ANTERIOR + O VALOR DO LANAMENTO */
      V_VRSALDATUAL := V_VRSALDATUAL + C_LANC.VRLANCCUST;
      IF C_LANC.VRLANCCUST > 0 THEN
         V_VRDEBITO := 0;
         V_VRCREDITO := C_LANC.VRLANCCUST;
      ELSE
         V_VRDEBITO := C_LANC.VRLANCCUST;
         V_VRCREDITO := 0;
      END IF;
      V_NRSEQ := SEQUENCIAL;
      INSERT INTO RAZAOCOMFIL
         (NRRAZCOMFIL, NRSEQRAZCOM, CDEMPRESA, DTCOMPCUSTO, NRLANCCONT, NRSEQLANC, CDFILIAL,
          CDCONTCTBL, CDCONTCOME, IDREGISTRO, IDTIPOCOME, VRSALDANT,
          VRDEBITO, VRCREDITO, VRSALDATUAL)
      VALUES
          (V_NRRAZAO, V_NRSEQ, P_CDEMPRESA, C_LANC.DTCOMPCUSTO, C_LANC.NRLANCCONT, C_LANC.NRSEQLANC, C_LANC.CDFILIAL,
           C_LANC.CDCONTCTBL, C_LANC.CDCONTCOME, 'L', C_LANC.IDTIPO, V_VRSALDANT,
           V_VRDEBITO, V_VRCREDITO, V_VRSALDATUAL);

      /* ATUALIZA SALDO ANTERIOR */
      V_VRSALDANT := V_VRSALDANT + C_LANC.VRLANCCUST;
   END LOOP;
   P_NRRAZAO := V_NRRAZAO;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('RAZAO_AUX_FIL','5.85.000'); END;
/